local M = {}

local emergencyDoorAlarmSound = nil
local sfxprofilecounter = 0 -- local counter to enumerate the profiles without collisions, do not reset ever

local lastCamPos = nil
local isCameraInside = false

local emergencyDoorHingeCid = -1
local emergencyDoorLatchCid = -1
local driverNodeCid = -1

local hasDash = false
local hasDoors = false

local doorOpenSound = nil
local doorCloseSound = nil
local timeSinceDoorSoundPlayed = 0
local lastDoorValue = false
local doorSoundPlaying = false
local doorBeamCid = nil

local soundObj = {}
soundObj.__index = soundObj

abs = math.abs
min = math.min
max = math.max

function soundObj:setVolumePitch(vol, pitch, color, texture)
  if vol < 0.01 then
    vol = 0
  end
  if vol == 0 then
    if self.lastVol == 0 then
      return
    end
    color, texture = color or 0, texture or 0
  else
    color, texture = color or 0, texture or 0
    if abs(pitch - self.lastPitch) < 0.001 and max(abs(vol - self.lastVol), abs(color - self.lastColor), abs(texture - self.lastTexture)) < 0.01 then
      return
    end
  end
  self.lastVol = vol
  self.lastPitch = pitch
  self.lastColor = color
  self.lastTexture = texture
  obj:setVolumePitchCT(self.obj, vol, pitch, color, texture)
end

function soundObj:mute()
  self:setVolumePitch(0, self.lastPitch)
end

function soundObj:unmute()
  self:setVolumePitch(1, self.lastPitch)
end

function soundObj:setVolumeOP(vol)
   --setVolume One Pitch
  self:setVolumePitch(vol, 1.0)
end

local function newSoundObj(sndObj)
  if sndObj == nil then
    return nil
  end
  local data = {obj = sndObj, lastVol = 0, lastPitch = 0, lastColor = 0, lastTexture = 0}
  setmetatable(data, soundObj)
  return data
end

local function createSoundObj(filename, description, SFXProfileName, nodeID)
  local snd = obj:createSFXSource(filename, description, SFXProfileName, nodeID)
  if snd == nil then
    log("W", "bus_custom_sounds.createSoundObj", "failed to create sfx source: " .. SFXProfileName .. " from file " .. filename .. " with description " .. description)
    return nil
  end
  return newSoundObj(snd)
end


local function getNextProfile()
  sfxprofilecounter = sfxprofilecounter + 1
  return "VANBUS_LuaSoundProfile" .. sfxprofilecounter .. "_" .. os.time()
end

local function findBeamCid(name)
  for _,beam in pairs(v.data.beams) do
      if beam.name == name then return beam.cid end
  end
  return -1
end
local function findNodeCid(name)
  for _,node in pairs(v.data.nodes) do
      if node.name == name then return node.cid end
  end
  return -1
end

local function onInit()
    driverNodeCid = findNodeCid("driver")
    emergencyDoorHingeCid = findBeamCid("emergencyDoorHinge")
    emergencyDoorLatchCid = findBeamCid("emergencyDoorLatch")
    doorBeamCid = findBeamCid("doorHydro")
    
    local dashNodeCid = findNodeCid("dsh1")
    if dashNodeCid < 0 then dashNodeCid = findNodeCid("p2ll") end
    
    if dashNodeCid >= 0 then
      emergencyDoorAlarmSound = emergencyDoorAlarmSound or createSoundObj("vehicles/van/dm_bus/custom_sounds/emerg_exit_alarm.wav", "AudioDefaultLoop3D", getNextProfile(), dashNodeCid)
      emergencyDoorAlarmSound:setVolumePitch(0, 1)
      hasDash = true
    end
   
    local doorSoundNode = findNodeCid("bd1drr")
    local doorNode = findNodeCid("drflnk")
    if doorNode >= 0 then
      doorOpenSound = obj:createSFXSource( "event:>Vehicle>Pneumatics>Door_Open", "AudioDefaultLoop3D", "airdooropen", doorSoundNode)
      doorCloseSound = obj:createSFXSource( "event:>Vehicle>Pneumatics>Door_Close", "AudioDefaultLoop3D", "airdoorclose", doorSoundNode)
      hasDoors = true
    end
end

local function stopDoorSounds()
    obj:stopSFX(doorCloseSound)
    obj:stopSFX(doorOpenSound)
    doorSoundPlaying = false
end

local function onReset()
  isCameraInside = false
  lastDoorValue = false
  
  if hasDoors then
    timeSinceDoorSoundPlayed = 999 --make sound playable again
    stopDoorSounds()
  end
end

local function getSideDoorsOpen()
    return (electrics.values["open_doors"] or 0) > 0.5
end

local function shouldPlayEmergencyDoorAlarm()
  return obj:beamIsBroken(emergencyDoorLatchCid) and not obj:beamIsBroken(emergencyDoorHingeCid)
end

local function updateCameraShit()
  local camPos = obj:getCameraPosition()
  local driverPos = obj:getNodePosition(driverNodeCid)
  local myPos = obj:getPosition()
  local combinedPos = myPos + driverPos
  
  local camDistFromDriver = (combinedPos - camPos):length()
  isCameraInside = camDistFromDriver <= 0.6
end

local function updateGFX(dt)
  updateCameraShit()

  -- update door stuff  
  if hasDoors then
    timeSinceDoorSoundPlayed = timeSinceDoorSoundPlayed + dt
    local doorValue = getSideDoorsOpen()
    if timeSinceDoorSoundPlayed >= 2  and doorSoundPlaying then
        stopDoorSounds()
    end
    
    local doorBrokenOff = false
    if doorBeamCid ~= nil then doorBrokenOff = obj:beamIsBroken(doorBeamCid) end
    
    if doorValue ~= lastDoorValue and timeSinceDoorSoundPlayed > 2 and not doorBrokenOff then
        if doorValue then
            obj:setVolume(doorOpenSound, 1)
            obj:cutSFX(doorOpenSound)
            obj:playSFX(doorOpenSound)
        else
            obj:setVolume(doorCloseSound, 1)
            obj:cutSFX(doorCloseSound)
            obj:playSFX(doorCloseSound)
        end
        doorSoundPlaying = true
        timeSinceDoorSoundPlayed = 0
    end
    lastDoorValue = doorValue
  end
  
  -- interior sounds
  if hasDash then
    if not isCameraInside then
      emergencyDoorAlarmSound:setVolumeOP(0)
      return
    end
    
    if shouldPlayEmergencyDoorAlarm() then
      emergencyDoorAlarmSound:setVolumeOP(0.25)
    else
      emergencyDoorAlarmSound:setVolumeOP(0)
    end
  end
end

-- public interface
M.onInit      = onInit
M.onReset     = onReset
M.updateGFX = updateGFX

M.cameraPosCallback = cameraPosCallback

return M